@@ -23,8 +23,6 @@ done
 
 #echo "Running command: @MATLAB_MEX_EXE@ $mexargs"
 
-@MATLAB_MEX_EXE@ $mexargs
-
- 
+"@MATLAB_MEX_EXE@" $mexargs
 
  
\ No newline at end of file
