@@ -40,14 +40,22 @@
 #include <getopt.h>
 #include <string.h>
 
+// Because non-free OpenCV modules is not included in most of the distributions
+// the current codes relies now on a less restrictive AKAZE feature detector
+// Uncomment the line below to use SURF
+//#define USE_SURF
+
 #include <opencv2/core/core.hpp>
 #include <opencv2/highgui/highgui.hpp>
 #include <opencv2/features2d/features2d.hpp>
-#include <opencv2/nonfree/features2d.hpp>
 #include <opencv2/calib3d/calib3d.hpp>
 #include <opencv2/imgproc/imgproc.hpp>
 #include <libexif/exif-data.h>
 
+#ifdef USE_SURF
+    #include <opencv2/nonfree/features2d.hpp>
+#endif
+
 #include <pfs.h>
 
 #define PROG_NAME "pfsalign"
@@ -71,22 +79,28 @@ void printHelp()
 class Toc
 {
     double tickCount;
+    bool newLine;
+    const char *name;
 public:
-    void tic( const char *name = NULL )
+    void tic( const char *name = NULL, bool newLine = false )
     {
-        if( name != NULL ) {
-            VERBOSE_STR << "Starting " << name << "...";
-        } else {
-            VERBOSE_STR << "Starting processing ...";
-        }
+        this->name = name;
+        this->newLine = newLine;
+        if( name != NULL ) 
+            name = "processing";
+        VERBOSE_STR << "Starting " << name << "...";
+        if( newLine )
+            VERBOSE_STR << std::endl;
         tickCount = static_cast<double>( getTickCount() );
     }
 
     void toc( )
     {
         double duration = (static_cast<double>( getTickCount() ) - tickCount) / getTickFrequency();
-        if( verbose )
-            std::cerr << "completed. It took " << duration << " seconds." << std::endl;
+        if( newLine ) {
+            VERBOSE_STR << "Completed " << name << ". It took " << duration << " seconds." << std::endl;
+        } else
+            VERBOSE_STR << "completed. It took " << duration << " seconds." << std::endl;            
     }
 };
 
@@ -129,30 +143,47 @@ void symmetryTest( const vector<vector<DMatch> > &matches_12, const vector<vecto
 }
 
 
+
+
 /**
  Match feature points in a pair image and find homography.
   */
 bool alignImagePair( const Mat &ref_img, const Mat &exp_img, Mat &homography, int sensitivity, bool display_matches )
 {
     Toc toc;
+    Toc toc_global;
+
+    toc_global.tic( "alignment of image pair", true );
 
     homography = Mat::eye( 3, 3, CV_64F );
     //    cv::imshow( "Result", ref_img );
     //    cv::imshow( "Result2", exp_img );
     //    cv::waitKey(0);
 
-    Ptr<FeatureDetector> detector(new DynamicAdaptedFeatureDetector( new SurfAdjuster( (11-sensitivity) * 100.f, 2, 1000 ),
-                                                                     100, 1000, sensitivity/2+2 ));
 //    Ptr<FeatureDetector> detector;
     //    detector = new GoodFeaturesToTrackDetector();
 //    detector = new SurfFeatureDetector();
 
     std::vector<KeyPoint> keypoints1, keypoints2;
+    Mat descr_ref, descr_exp;
 
-    toc.tic( "feature detection" );
-    detector->detect( ref_img, keypoints1 );
-    detector->detect( exp_img, keypoints2 );
-    toc.toc( );
+    #ifdef USE_SURF
+        Ptr<FeatureDetector> detector(new DynamicAdaptedFeatureDetector( new SurfAdjuster( (11-sensitivity) * 100.f, 2, 1000 ),
+                                                                     100, 1000, sensitivity/2+2 ));
+        toc.tic( "feature detection" );
+        detector->detect( ref_img, keypoints1 );
+        detector->detect( exp_img, keypoints2 );
+        toc.toc( );
+    #else
+        Ptr<AKAZE> akaze = AKAZE::create();
+        toc.tic( "feature detection and extraction" );
+        akaze->setThreshold( 0.005f );        
+        akaze->detectAndCompute( ref_img, noArray(), keypoints1, descr_ref);
+        akaze->detectAndCompute( exp_img, noArray(), keypoints2, descr_exp);
+        toc.toc( );
+        VERBOSE_STR << "Detected " << keypoints1.size() << " keypoints in the reference image." << endl;
+        VERBOSE_STR << "Detected " << keypoints2.size() << " keypoints in the test image." << endl;
+    #endif
 
     if( keypoints1.size() < 10 || keypoints2.size() < 10 ) {
         cerr << PROG_NAME ": Could not detect a sufficient number of keypoints (found "
@@ -172,22 +203,25 @@ bool alignImagePair( const Mat &ref_img, const Mat &exp_img, Mat &homography, in
     }
 
     //        SiftDescriptorExtractor surfDesc;
-    SurfDescriptorExtractor surfDesc;
+    #ifdef USE_SURF
+        SurfDescriptorExtractor surfDesc;
 
-    Mat descr_ref, descr_exp;
-    toc.tic( "descriptor extraction" );
-    surfDesc.compute( ref_img, keypoints1, descr_ref );
-    surfDesc.compute( exp_img, keypoints2, descr_exp );
-    toc.toc( );
-
-    FlannBasedMatcher matcher;
-    //        BruteForceMatcher< cv::L2<float> > matcher;
+        toc.tic( "descriptor extraction" );
+        surfDesc.compute( ref_img, keypoints1, descr_ref );
+        surfDesc.compute( exp_img, keypoints2, descr_exp );
+        toc.toc( );
+    #endif
 
+    #ifdef USE_SURF
+        FlannBasedMatcher matcher;
+        //        BruteForceMatcher< cv::L2<float> > matcher;
+    #else
+        BFMatcher matcher(NORM_HAMMING);
+    #endif
     toc.tic( "matching" );
     std::vector< std::vector<cv::DMatch> > matches_rt;
-    matcher.knnMatch( descr_ref, descr_exp, matches_rt, 2 );
-
     std::vector< std::vector<cv::DMatch> > matches_tr;
+    matcher.knnMatch( descr_ref, descr_exp, matches_rt, 2 );
     matcher.knnMatch( descr_exp, descr_ref, matches_tr, 2 );
 
     pruneNNDR( matches_rt, 1 );
@@ -226,7 +260,7 @@ bool alignImagePair( const Mat &ref_img, const Mat &exp_img, Mat &homography, in
 
     vector<uchar> inliners(matches_sym.size(), 0);
     //        affine = findHomography( pp1, pp2, inliners, CV_RANSAC, 1. );
-    homography = findHomography( pp2, pp1, CV_RANSAC, 1., inliners );
+    homography = findHomography( pp2, pp1, RANSAC, 1., inliners );
 
     //    solve( pp1, pp2, affine, DECOMP_SVD );
     //    Mat affine = (Mat_<float>(2,3) << 1, 0, 0, 0, 1, 10);
@@ -253,6 +287,7 @@ bool alignImagePair( const Mat &ref_img, const Mat &exp_img, Mat &homography, in
         cerr << PROG_NAME ": Not enough inliners to find a reliable transformation." << endl;
         return false;
     }
+    toc_global.toc();
 
 
     return true;
@@ -643,7 +678,7 @@ void alignFrames(int argc, char *argv[])
 
         homoMC.push_back( trans );
 
-        double data[4][3] = { { 0, 0, 1 }, { frames[kk].size.width, 0, 1 }, { frames[kk].size.width, frames[kk].size.height, 1 }, { 0, frames[kk].size.height, 1 } };
+        double data[4][3] = { { 0., 0., 1. }, { (double)frames[kk].size.width, 0., 1. }, { (double)frames[kk].size.width, (double)frames[kk].size.height, 1. }, { 0., (double)frames[kk].size.height, 1. } };
         Mat corners( 4, 3, CV_64F, data );
 
         Mat corners_trans = trans * corners.t();
