@@ -31,6 +31,7 @@
 #include <config.h>
 
 #include <iostream>
+#include <sstream>
 #include <vector>
 
 #include <stdio.h>
@@ -44,6 +45,8 @@
 #include <robertson02.h>
 #include <mitsunaga99.h>
 
+#include <string>
+
 using namespace std;
 
 #define PROG_NAME "pfshdrcalibrate"
@@ -83,6 +86,9 @@ void printHelp()
     "\t[--save-response <filename.m>] \n"
     "\t[--multiplier <val>] \n"
     "\t[--bpp <val>] \n"
+    "\t[--saturation-offset <val>] \n"
+    "\t[--deghosting <val>] \n"
+    "\t[--noise-parameters <val>] \n"
     "\t[--verbose] [--help]\n"
     "See man page for more information.\n" );
 }
@@ -106,7 +112,14 @@ void pfshdrcalibrate( int argc, char* argv[] )
     { FROM_FILE, LINEAR, GAMMA, LOG10 } opt_response = LINEAR;
   enum TMethod
     { ROBERTSON_METHOD, MITSUNAGA_METHOD } opt_method = ROBERTSON_METHOD;
-	
+
+  noise_parameters camera_presets[] = {
+    { "Empty", {1, 1, 1}, 0, 0 },
+    { "SonyA7r1", {1.308256, 1.321364, 1.28384}, 2.796408, 0.1434412 },
+    { "CanonT1", {5.45232, 4.73024, 4.61276}, 3.752112, 20.021 },
+    { "SonyA7r3", {1.688176, 1.537108, 1.555868}, 2.820508, 12.1138 }
+  };
+
   /* defaults */
   float         input_multiplier              = 1.0f;
   FILE         *responseFile                  = NULL;
@@ -118,8 +131,11 @@ void pfshdrcalibrate( int argc, char* argv[] )
   int           opt_maxresponse               = -1;
   int           opt_minresponse               = -1;
   unsigned long mitsunaga_sample_no           = MITSUNAGA_SAMPLES_NO;
-  bool		opt_deghosting = false;
-  
+  int           opt_saturation_offset         = -1;
+  float         opt_saturation_offset_perc    = -1;
+  float         opt_deghosting                = -1;
+  noise_parameters *camera                    = &camera_presets[0];
+
   /* helper */
   int c;
 
@@ -137,14 +153,16 @@ void pfshdrcalibrate( int argc, char* argv[] )
     { "multiplier", required_argument, NULL, 'm' },
     { "bpp", required_argument, NULL, 'b' },
     { "samples", required_argument, NULL, 'p' },	
-    { "deghosting", no_argument, NULL, 'd' },	
+    { "saturation-offset", required_argument, NULL, 'o' },
+    { "deghosting", required_argument, NULL, 'd' },
+    { "noise-parameters", required_argument, NULL, 'n'},
     { NULL, 0, NULL, 0 }
   };
 
   int optionIndex = 0;
   while( ( c =  getopt_long( argc, 
 			     argv, 
-			     "hvYc:g:r:f:s:m:b:p:xd", 
+			     "hvYc:g:r:f:s:m:b:p:xo:d:n:",
 			     cmdLineOptions, 
 			     &optionIndex) ) 
 	 != -1 ) 
@@ -267,11 +285,62 @@ void pfshdrcalibrate( int argc, char* argv[] )
 	  if( opt_minresponse<0 )
 	    throw pfs::Exception("min response should be >0");
 	  break;
+
+  case 'o':
+    opt_saturation_offset_perc = atof(optarg);
+    if( opt_saturation_offset_perc < 0 || opt_saturation_offset_perc > 0.5 )
+      throw pfs::Exception("saturation offset should be between 0 and 0.5");
+    break;
 	  
 	case 'd':
-		opt_deghosting = true;
+    if( !optarg )
+      opt_deghosting = 2;
+		else
+      opt_deghosting = atof(optarg);
+    if( opt_deghosting<0 )
+      throw pfs::Exception("deghosting threshold should be >0");
 		break;
 
+  case 'n':
+  {
+    int i = 0;
+    if( isalpha(optarg[0]) )
+    {
+      // Name of a camera, whose noise parameters are known, is provided
+      for( int i = 0; i < sizeof(camera_presets)/sizeof(*camera_presets); i++ )
+        if( strcmp( optarg, camera_presets[i].camera_name ) == 0 )
+          camera = &camera_presets[i];
+
+      if( strcmp( optarg, "Empty" ) != 0 && strcmp( camera->camera_name, "Empty" ) == 0 )
+        throw pfs::Exception("specified camera noise parameters not found");
+    }
+    else if( isdigit(optarg[0]) )
+    {
+      // Custom noise parameters provided as a comma separated list
+      stringstream sstream(optarg);
+      string token;
+      for( int cc = 0; cc < 3; cc++ )
+      {
+        camera->camera_name = "Custom";
+        if( !getline(sstream, token, ',') )
+          throw pfs::Exception("invalid noise parameters");
+        camera->color_coefficients[cc] = atof(token.c_str());
+      }
+      if( !getline(sstream, token, ',') )
+        throw pfs::Exception("invalid noise parameters");
+      camera->std_readout = atof(token.c_str());
+      if( !getline(sstream, token, ',') )
+        throw pfs::Exception("invalid noise parameters");
+      camera->std_adc = atof(token.c_str());
+
+      if( getline(sstream, token, ',') )
+        throw pfs::Exception("invalid noise parameters");
+    }
+    else
+      throw pfs::Exception("invalid noise parameters");
+    break;
+  }
+
 	case '?':
 	  throw QuietException();
 
@@ -334,7 +403,13 @@ void pfshdrcalibrate( int argc, char* argv[] )
       break;
     }
   
-	
+
+if( opt_saturation_offset_perc==-1 ) {
+  // By default, use 1% of the maximum value as the saturation offset (pixels above 99 of the max pixel value will be considered as saturated)
+  opt_saturation_offset_perc = 0.01;
+}
+opt_saturation_offset = (int) (powf( 2.0f, opt_bpp )*opt_saturation_offset_perc + 0.5);
+
 
 //   VERBOSE_STR << "interpolate missing parts of response: "
 //               << (opt_fillgaps ? "yes" : "no") << endl;
@@ -799,7 +874,7 @@ void pfshdrcalibrate( int argc, char* argv[] )
               const float *resp_curves[] = { Ir, Ig, Ib };
               
               VERBOSE_STR << "applying response to RGB channels..." << endl;
-	      sp = robertson02_applyResponseRGB( RGB_out, exposures, resp_curves, w, M, opt_deghosting );
+	      sp = robertson02_applyResponseRGB( RGB_out, exposures, resp_curves, w, M - opt_saturation_offset, camera, opt_deghosting);
 
 /*                                            
               VERBOSE_STR << "applying response to R channel..." << endl;
