@@ -40,6 +40,7 @@
 #include <robertson02.h>
 
 #include <iostream>
+#include <cstring>
 
 using namespace std;
 
@@ -56,19 +57,10 @@ extern bool verbose; /* verbose should be declared for each standalone code */
 
 float normalize_rcurve( float *rcurve, int M );
 
-float corr_vec3( float *a, float *b )
+inline float max3( float a[3])
 {
-  float ma = (a[0] + a[1] + a[2])/3.f;
-  float mb = (b[0] + b[1] + b[2])/3.f;
-
-  float sa = 1e-5, sb = 1e-5, cov = 0;
-  for( int cc = 0; cc < 3; cc++ ) {
-    sa += powf(a[cc] - ma, 2.f);    
-    sb += powf(b[cc] - mb, 2.f);
-    cov += (a[cc] - ma)*(b[cc] - mb);
-  }
-
-  return cov / sqrt(sa * sb);  
+  float max = (a[0]>a[1]) ? a[0] : a[1];
+  return (a[2]>max) ? a[2] : max;
 }
 
 float get_exposure_compensation( const Exposure &ex )
@@ -81,7 +73,8 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
   const float *resp_curve[], 
   const float *weights, 
   int M,
-  bool deghosting )
+  const noise_parameters *camera,
+  const float deghosting_threshold)
 {
   // number of exposures
   int N = imgs[0]->size();
@@ -93,6 +86,9 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
   // number of saturated pixels
   int saturated_pixels = 0;
 
+  float mmax[3]= {1e-30, 1e-30, 1e-30};
+
+/*
   float pw[M];
   for( int i = 0 ; i < M ; i++ ) {
     pw[i] = weights[i];
@@ -101,6 +97,7 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
     pw[M-i] = 0;
   
   float ew[N]; // Per-exposure weight to account for noise charactetistic
+
   // For each exposure
   for( int i = 0 ; i < N ; i++ ) {
     const Exposure &ex = (*imgs[0])[i];
@@ -115,56 +112,105 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
     VERBOSE_STR << "Ex: " << i << " exp. time: " << ex.exposure_time << "\t iso: " << ex.iso 
                 << "\t aperture: " <<  ex.aperture << "\t weight: " << ew[i] << std::endl;
   }
+*/
 
-  float X[3*N];
-  bool skip_exp[N];
-  float w[3][N];
-  bool saturated_exp[N];  
+  bool photon_noise_only = false;
+  if( strcmp( camera->camera_name, "Empty" ) == 0 )
+  {
+    photon_noise_only = true;
+    VERBOSE_STR << "Merging using the Poisson Photon Noise Estimator" << endl;
+  }
+  else
+    VERBOSE_STR << "Merging using the Variance Weighted Estimator" << endl;
 
-//  for( int i = 0; i < M; i++ )
-//    cerr << "i = " << i << "; weight = " << weights[i] << endl;  
-  
   // For each pixel
   int skipped_deghost = 0;
   for( int j = 0; j < width * height; j++ )
   {
-    
     bool saturatedRGB[3] = { false, false, false };
 
-    float best_mw = -1;  
-    int ref = -1;    
-	
-    if( deghosting ) {
-      
-      // Deghosting - find the exposure that could be used as a reference
-      for( int i = 0 ; i < N ; i++ )
+    // First compute scene radiances, weights and saturated exposures
+    float X[3][N];
+    float w[3][N];
+    bool saturated_exp[N];
+    fill(saturated_exp, saturated_exp + N, false);
+    bool skip_exp[N];
+    fill(skip_exp, skip_exp + N, false);
+    float t[N];
+    float g[N];
+
+    for( int i = 0; i < N; i++ )
+      for( int cc = 0; cc < 3; cc++ )
       {
-        saturated_exp[i] = false;
-        for( int cc=0; cc < 3; cc++ )
+        const Exposure &ex = (*imgs[cc])[i];
+        t[i] = ex.exposure_time;
+        g[i] = ex.iso / 100;
+      
+        int   m  = (int) (*ex.yi)( j );
+        X[cc][i] = resp_curve[cc][ m ] / ( t[i] * g[i] * camera->color_coefficients[cc] );
+        if( photon_noise_only )
+          w[cc][i] = t[i];
+        else
         {
-          const Exposure &ex = (*imgs[cc])[i];          
-          float ti = get_exposure_compensation( ex );
-        
-          int   m  = (int) (*ex.yi)( j );
-          X[i*3+cc] = resp_curve[cc][ m ] / ti;
-          w[cc][i] = weights [ m ] * ew[i];
-
-          saturated_exp[i] |= ( m <= 1 || m >= M-2 );        
-        
-//        cerr << X[i*3+cc] << ", ";        
-
+          float var = resp_curve[cc][ m ] * g[i] * camera->color_coefficients[cc]
+                      + powf(camera->std_readout * g[i] * camera->color_coefficients[cc], 2)
+                      + powf(camera->std_adc * camera->color_coefficients[cc], 2);
+            w[cc][i] = powf( t[i] * g[i] * camera->color_coefficients[cc], 2 ) / var;
         }
-        skip_exp[i] = saturated_exp[i];      
-	  
-        float mw = (w[0][i] + w[1][i] + w[2][i])/3.f;
-        if( mw > best_mw && !saturated_exp[i] ) {
-          best_mw = mw;
-          ref = i;        
-        }      
-//      cerr << " ; ";      
+
+        // Test for overexposed pixels
+        saturated_exp[i] |= resp_curve[cc][m] >= resp_curve[cc][M-1];
       }
+
+    // Deghosting - find the exposure that could be used as a reference
+    // Currently using middle exposure
+    // TODO: Perform histogram analysis to pick best reference
+    int ref = int(N/2);
+    if( deghosting_threshold != -1 )
+    {
+      // Compute the number of pixels to be skipped due to deghosting
+      for( int i = 0; i < N; i++ )
+        if(i != ref && !saturated_exp[ref] && !saturated_exp[i])
+        {
+          // First compute the Pearson correlation coefficient wrt reference
+          float ref_mean = 0, ref_var = 1e-5, cur_mean = 0, cur_var = 1e-5, cov = 0;
+          for( int cc = 0; cc < 3; cc++ )
+          {
+            ref_mean += X[cc][ref];
+            cur_mean += X[cc][i];
+          }
+          ref_mean /= 3;
+          cur_mean /= 3;
+          for( int cc = 0; cc < 3; cc++ )
+          {
+            ref_var += powf(X[cc][ref] - ref_mean, 2);
+            cur_var += powf(X[cc][i] - cur_mean, 2);
+            cov += (X[cc][ref] - ref_mean)*(X[cc][i] - cur_mean);
+          }
+
+          // Next check whether total deviation from the reference is acceptable
+          float std_sum = 0;
+          float deviation_from_ref = 0;
+          for( int cc = 0; cc < 3; cc++ )
+          {
+            // Reciprocal of the weight is the exposure compensated variance
+            // Square root of this is thus the exposure compensated standard deviation
+            // Assuming a gaussian distribution, 95% of the population should lie 2 stds away from mean
+            std_sum += deghosting_threshold * sqrt(1/w[cc][ref] + 1/w[cc][i]);
+            deviation_from_ref += fabs( (float)(X[cc][ref] - X[cc][i]) );
+          }
+
+          // Skipping the Pearson coefficient test as reults are not satisfactory
+          // Fixing a better reference might solve this
+          // float rho = cov / sqrt(ref_var/2 * cur_var/2);
+          // skip_exp[i] = rho < 0.9 || deviation_from_ref > std_sum;
+          skip_exp[i] = deviation_from_ref > std_sum;
+          if( skip_exp[i] )
+            skipped_deghost++;
+        }
+    }
     	
-      // Deghosting
+/*     // Deghosting
       if( ref != -1 && !saturated_exp[ref] ) {
 
         const Exposure &ex = (*imgs[0])[ref];          
@@ -203,22 +249,22 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
             skipped_deghost++;
         }
       }
-    }
     
     
-/*    int skip_bits = 0;
+      int skip_bits = 0;
     
       for( int i = 0 ; i < N ; i++ ) {
 
       if( !skip_exp[i] )
       skip_bits++; // |= (1<<i);
-      }*/
+      }
     
 
     
-/*   for( int i = 0 ; i < N ; i++ ) {        
+     for( int i = 0 ; i < N ; i++ ) {        
      skip_exp[i] = false;
-     }*/
+     }
+*/
       
     
 //   cerr << endl;    
@@ -226,77 +272,34 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
     // For each color channels
     for( int cc=0; cc < 3; cc++ )
     {
-        
+
       // all exposures for each pixel
       float sum = 0.0f;
       float div = 0.0f;
-      float div2 = 0.0f;
 
       // For each exposure
       for( int i = 0 ; i < N ; i++ )
       {
-        if( deghosting && (skip_exp[i] || saturated_exp[i]) )
+        if( saturated_exp[i] || (deghosting_threshold != -1 && skip_exp[i]) )
           continue;
-        
-        const Exposure &ex = (*imgs[cc])[i];          
-        int   m  = (int) (*ex.yi)( j );
-        float ti = get_exposure_compensation( ex );
 
-        // Testing the noise-optimal approach
-/*        float iso2 = ex.iso*ex.iso;
-          float sigma_r2 = 0.00001;
-          float w = ex.exposure_time*ex.exposure_time*iso2/(iso2*ex.exposure_time*resp_curve[cc][m] + 2*sigma_r2);        
-          sum += w * resp_curve[cc][ m ] / ti;
-          div += w;*/
-        
-//        float w = weights [ m ] * ew[i];
-//        sum += w * resp_curve[cc][ m ] / ti;
-//        div += w;
+        sum += X[cc][i] * w[cc][i];
+        div += w[cc][i];
 
-//        const float noise_level = 0.01;      
-//        float var = (noise_level*noise_level)*resp_curve[cc][ m ]/(ti*ti);
-        
-//        float w = 1/var;
-//        float w = log(sqrt(resp_curve[cc][ m ] / ti / var))+3;
-//        float w = log(sqrt(m));
-        float w = weights [ m ] * ew[i];
-        
-        sum += w * resp_curve[cc][ m ] / ti;
-        div += w;
-
-        if( j < 10 && cc == 1 )
-          std::cerr << "j = " << j << " ;i = " << i << " ;w = " << w << std::endl;        
-        
-        div2 += weights [ m ];        
+//        if( j < 10 && cc == 1 )
+//         std::cerr << "j = " << j << " ;i = " << i << " ;w = " << w[cc][i] << std::endl;
       }
 
-/*      float mean = sum / div;
-
-        sum = 0;
-        div = 0;      
-        // For each exposure
-        for( int i = 0 ; i < N ; i++ )
-        {
-        const Exposure &ex = (*imgs[cc])[i];          
-        int   m  = (int) (*ex.yi)( j );
-        float ti = ex.ti;
-
-        float ww = 1/(fabs(resp_curve[cc][ m ]/ti - mean) + 1e-7);
-        
-        sum += weights [ m ] * ew[i] * resp_curve[cc][ m ] * ww / ti;
-        div += weights [ m ] * ew[i] * ww;
-        }*/
-      
-      if( div2 >= 0.0001f )
-	(*rgb_out[cc])(j) = sum / div;
-      else {
+      if( div >= 1e-15 )
+      {
+        (*rgb_out[cc])(j) = sum / div;
+        mmax[cc] = ( mmax[cc] > (*rgb_out[cc])(j) ) ? mmax[cc] : (*rgb_out[cc])(j);
+      }
+      else
+      {
+        (*rgb_out[cc])(j) = -1;
         saturatedRGB[cc] = true;
       }
-
-//      saturatedRGB[cc] = false;
-//      (*rgb_out[cc])(j) = saturated_exp[ref];      
-      
-      
     }
     
 
@@ -304,7 +307,7 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
       saturated_pixels++;      
     }
     
-    for( int cc=0; cc < 3; cc++ ) {
+/*  for( int cc=0; cc < 3; cc++ ) {
       // TODO: Some fancy stuff to deal with distorted colors
       
       if( saturatedRGB[cc] ) {        
@@ -334,10 +337,20 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
         (*rgb_out[cc])(j) = 1/best_ti * resp_curve[cc][best_v];
       }
       
-    }
+    }*/
     
     
   }
+
+  // Fill in nan values and normalise
+  for( int j = 0; j < width * height; j++ )
+    for( int cc=0; cc < 3; cc++ )
+    {
+      if( (*rgb_out[cc])(j) == -1)
+        (*rgb_out[cc])(j) = mmax[cc];
+      (*rgb_out[cc])(j) /= max3(mmax);
+    }
+
   VERBOSE_STR << "Exposure pixels skipped due to deghosting: " << 
     (float)skipped_deghost*100.f/(float)(width*height*N) << "%" << std::endl;
 
