@@ -31,6 +31,15 @@
 
 #include <responses.h>
 
+struct noise_parameters
+{
+  const char* camera_name;
+  float color_coefficients[3];
+  float std_readout;
+  float std_adc;
+};
+
+
 /**
  * @brief Calculate camera response using Robertson02 algorithm
  *
@@ -72,7 +81,8 @@ int robertson02_applyResponseRGB( pfs::Array2D *rgb_out[],
   const float *resp_curve[], 
   const float *weights, 
   int M,
-  bool deghosting = false );
+  const noise_parameters *camera,
+  const float deghosting_value = -1);
 
   
 #endif /* #ifndef _robertson02_h_ */
