@@ -22,7 +22,7 @@
  * 
  * @author Grzegorz Krawczyk, <krawczyk@mpi-sb.mpg.de>
  *
- * $Id: hdrtiffio.cpp,v 1.5 2011/10/27 08:39:07 rafm Exp $
+ * $Id: hdrtiffio.cpp,v 1.7 2014/01/03 12:52:20 rafm Exp $
  */
 
 #include <config.h>
@@ -36,6 +36,7 @@
 
 #include "hdrtiffio.h"
 
+using namespace std;
 
 int TIFFWriteIFD(FILE* file, unsigned short tag, unsigned short type,
 		 unsigned long count, unsigned long value);
@@ -142,13 +143,13 @@ HDRTiffReader::HDRTiffReader( const char* filename )
       break;
     case PHOTOMETRIC_MINISBLACK: // HDR video camera format (grayscale)
       DEBUG_STR << "Photometric data: MINISBLACK (hdrv camera)" << endl;
-      if (!TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &bps) || !bps==1)
+      if (!TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &bps) || bps!=1)
       {
 	TIFFClose(tif);
 	throw pfs::Exception("TIFF: Unsupported samples per pixel for "
 			     "grayscale image");
       }
-      if (!TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &bps) || !bps==16)
+      if (!TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &bps) || bps!=16)
       {
 	DEBUG_STR << "Detected bits per sample: " << bps << endl;
 	TIFFClose(tif);
