@@ -113,8 +113,9 @@ void readPFMFileColor( FILE *fh, PFMHeader &header, float *R, float *G, float *B
 {
   const int lineSize = header.width*3;
   float *line = new float[lineSize];
-  
-  for( int l = 0; l < header.height; l++ ) {
+
+  // PFM images are stored in rows bottom to top
+  for( int l = header.height-1; l >= 0; l-- ) {    
     int read = fread( line, sizeof( float ), lineSize, fh );
     if( read != lineSize )
       throw pfs::Exception( "Unexpected EOF" );
@@ -140,7 +141,8 @@ void readPFMFileColor( FILE *fh, PFMHeader &header, float *R, float *G, float *B
 
 void readPFMFileGrayscale( FILE *fh, PFMHeader &header, float *Y )
 {
-  for( int l = 0; l < header.height; l++ ) {
+  // PFM images are stored in rows bottom to top
+  for( int l = header.height-1; l >= 0; l-- ) {    
     int read = fread( Y + header.width*l,
       sizeof( float ), header.width, fh );
 
