@@ -106,60 +106,65 @@ void readFrames( int argc, char* argv[] )
   VERBOSE_STR << "linearize input image: " << ((opt_linear || absoluteMaxLum!=0) ? "yes" : "no") << std::endl;
   if( absoluteMaxLum != 0 )
     VERBOSE_STR << "maximum absolute luminance: " << absoluteMaxLum << std::endl;
-
-  bool eofP = false;
-  while( !eofP ) {
+  
+  while( true ) {
     pfs::FrameFile ff = it.getNextFrameFile();
-    if( ff.fh == NULL ) break; // No more frames
+    if( ff.fh == NULL ) {
+      break; // No more frames
+    }    
 
     PPMReader reader( PROG_NAME, ff.fh );
+    bool eofP = false;
+    while( !eofP ) {
 
-    VERBOSE_STR << "reading file '" << ff.fileName << "'" << std::endl;
+      VERBOSE_STR << "reading file '" << ff.fileName << "'" << std::endl;
 
-    pfs::Frame *frame = pfsio.createFrame( reader.getWidth(),
-      reader.getHeight() );
-    pfs::Channel *X, *Y, *Z;
-    frame->createXYZChannels( X, Y, Z );
+      pfs::Frame *frame = pfsio.createFrame( reader.getWidth(),
+        reader.getHeight() );
+      pfs::Channel *X, *Y, *Z;
+      frame->createXYZChannels( X, Y, Z );
 
-    //Store sRGB data temporarily in XYZ channels        
-    eofP = reader.readImage( X, Y, Z );
-    if( opt_linear || absoluteMaxLum != 0 )
-    {
-      pfs::transformColorSpace( pfs::CS_SRGB, X, Y, Z, pfs::CS_XYZ, X, Y, Z );
-      if( absoluteMaxLum != 0 ) {
-        // Rescale to absolute luminance level
-        const int pixCount = X->getWidth()*X->getHeight();
-        for( int i = 0; i < pixCount; i++ ) {
-          (*X)(i) *= absoluteMaxLum;
-          (*Y)(i) *= absoluteMaxLum;
-          (*Z)(i) *= absoluteMaxLum;
-        }        
-        frame->getTags()->setString("LUMINANCE", "ABSOLUTE");
-      } else
-        frame->getTags()->setString("LUMINANCE", "RELATIVE");
-    }
-    else
-    {
-      pfs::transformColorSpace( pfs::CS_RGB, X, Y, Z, pfs::CS_XYZ, X, Y, Z );
-      frame->getTags()->setString("LUMINANCE", "DISPLAY");
-    }
+      //Store sRGB data temporarily in XYZ channels        
+      eofP = reader.readImage( X, Y, Z );
+      if( opt_linear || absoluteMaxLum != 0 )
+      {
+        pfs::transformColorSpace( pfs::CS_SRGB, X, Y, Z, pfs::CS_XYZ, X, Y, Z );
+        if( absoluteMaxLum != 0 ) {
+          // Rescale to absolute luminance level
+          const int pixCount = X->getWidth()*X->getHeight();
+          for( int i = 0; i < pixCount; i++ ) {
+            (*X)(i) *= absoluteMaxLum;
+            (*Y)(i) *= absoluteMaxLum;
+            (*Z)(i) *= absoluteMaxLum;
+          }        
+          frame->getTags()->setString("LUMINANCE", "ABSOLUTE");
+        } else
+          frame->getTags()->setString("LUMINANCE", "RELATIVE");
+      }
+      else
+      {
+        pfs::transformColorSpace( pfs::CS_RGB, X, Y, Z, pfs::CS_XYZ, X, Y, Z );
+        frame->getTags()->setString("LUMINANCE", "DISPLAY");
+      }
 
-    // This is the luminance / luma perceived as reference white
-    // Some tone-mappers may need this
-    frame->getTags()->setString("WHITE_Y", "1");
+      // This is the luminance / luma perceived as reference white
+      // Some tone-mappers may need this
+      frame->getTags()->setString("WHITE_Y", "1");
 
-    char strbuf[3];
-    snprintf( strbuf, 3, "%d", reader.getBitDepth() );
-    frame->getTags()->setString("BITDEPTH", strbuf );    
+      char strbuf[3];
+      snprintf( strbuf, 3, "%d", reader.getBitDepth() );
+      frame->getTags()->setString("BITDEPTH", strbuf );    
 
-    const char *fileNameTag = strcmp( "-", ff.fileName )==0 ? "stdin" : ff.fileName;
-    frame->getTags()->setString( "FILE_NAME", fileNameTag );
+      const char *fileNameTag = strcmp( "-", ff.fileName )==0 ? "stdin" : ff.fileName;
+      frame->getTags()->setString( "FILE_NAME", fileNameTag );
         
-    pfsio.writeFrame( frame, stdout );
-    pfsio.freeFrame( frame );
+      pfsio.writeFrame( frame, stdout );
+      pfsio.freeFrame( frame );
         
-    it.closeFrameFile( ff );
+      it.closeFrameFile( ff );  
+    }
   }
+  
 }
 
 
