@@ -101,11 +101,11 @@ void readFrames( int argc, char* argv[] )
     }
   }
 
-  if (!quiet && !radiance_compatibility)
+/*  if (!quiet && !radiance_compatibility)
 	  std::cerr << PROG_NAME << " warning: starting from pfstools 1.9.0, .hdr files are read without correcting for WHITE_EFFICIENCY,"
 	  " which makes the absolute values incompatible with Radiance and previos versions of pfstools but compatible with most software."
 	  " Use --radiance option to retain compatibility with Radiance and earlier versions of pfstools.  Use --quiet to suppress this message."
-	  " Check the manual pages for pfsinrgbe for details." << std::endl;
+	  " Check the manual pages for pfsinrgbe for details." << std::endl;*/
 
   while( true ) {
     pfs::FrameFile ff = it.getNextFrameFile();
