@@ -1,5 +1,5 @@
 /**
-az @brief Read RAW Yuv files, commonly used for video compression
+ * @brief Read RAW Yuv files, commonly used for video compression
  * 
  * This file is a part of PFSTOOLS package.
  * ---------------------------------------------------------------------- 
@@ -20,9 +20,8 @@ az @brief Read RAW Yuv files, commonly used for video compression
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  * ---------------------------------------------------------------------- 
  * 
- * @author Rafal Mantiuk, <mantiuk@mpi-sb.mpg.de>
+ * @author George Ash
  *
- * $Id: pfsinrgbe.cpp,v 1.5 2014/06/17 21:57:08 rafm Exp $
  */
 
 #include <config.h>
@@ -33,7 +32,9 @@ az @brief Read RAW Yuv files, commonly used for video compression
 #include <algorithm>
 #include <iostream>
 #include <vector>
+//#include <arpa/inet.h>
 
+#include <cmath>
 #include <getopt.h>
 #include <pfs.h>
 #include <pfsutils.cpp>
@@ -43,6 +44,9 @@ az @brief Read RAW Yuv files, commonly used for video compression
 
 char charsToRemove[7] = {'b', 'i', 't', 's', 'f', 'p', 's'};
 
+//PL means a planar format per frame.
+enum ChromaFormat {PL420, PL444, V210};
+
 template<typename T>
 inline T clamp( T val, T min, T max )
 {
@@ -79,83 +83,6 @@ bool read_yuv_channel( FILE *fh, int width, int height, int stride, pfs::Array2D
   return true;
 }
 
-void removeCharsFromString(std::string &str, char* charsToRemove ) {
-  for ( unsigned int i = 0; i < strlen(charsToRemove); ++i ) {
-        str.erase( remove(str.begin(), str.end(), charsToRemove[i]), str.end() );
-   }
-}
-
-int parseIntField(std::string i){
-  //parses an integer field. Removes pesky characters like the b in 10b or the p in 1080p
-  removeCharsFromString(i, charsToRemove);
-  return std::atoi(i.data());
-}
-
-//more readable helper method for strings, case insensitive
-bool contains(std::string s1, std::string s2){
-  
-  std::transform(s1.begin(), s1.end(), s1.begin(), ::tolower);
-  std::transform(s2.begin(), s2.end(), s2.begin(), ::tolower);
-  
-  return s1.find(s2) != std::string::npos || s1 == s2;
-}
-
-
-//returns a vector of strings tokenized 
-std::vector<std::string> split(std::string s, std::string delim){
-  return std::vector<std::string>();
-}
-
-bool parseFileName(const char * fileNameIn, int * width,  int * height, int * bitdepth, pfs::ColorSpace * colorspace, bool * upscale_420, int * fps){
-  try{
-    /* Parses the filename, puts the result into fields pointed to by the arguments
-     returns true if all went well, false if there's a malformed filename
-     string should be <prefix>_<width>x<height(p)>_<fps>_<bitdepth(b)>_<colorspace>_<chroma_format>
-     Regex would be nice*/  
-    std::string fileName(fileNameIn); 
-    const std::string delimiter = "_";
-    size_t pos = 0;
-    std::string token;  
-    std::vector<std::string> tokens;
-
-    while ((pos = fileName.find(delimiter)) != std::string::npos) {
-        token = fileName.substr(0, pos);
-        tokens.push_back(token);
-        fileName.erase(0, pos + delimiter.length());
-    }
-    tokens.push_back(fileName);
-
-    for (std::vector<string>::iterator tok = tokens.begin() + 1; tok != tokens.end(); ++tok) {
-      
-      if (contains(*tok, "x")){
-        sscanf(tok->c_str(), "%dx%d", width, height);
-      }
-      else if(*tok == "10" || *tok == "8" || *tok == "12" || tok->at(tok->length() - 1) == 'b' || contains(*tok, "bit")){ //last character is a b
-        *bitdepth = parseIntField(*tok); 
-      }
-      else if(*tok == "24" || *tok == "25" || *tok == "50" || *tok == "60" || contains(*tok, "fps")){
-        *fps = parseIntField(*tok);
-      }
-      else if (contains(*tok, "pq") || (contains(*tok, "2020") && !contains(*tok, "hlg"))){
-        *colorspace = pfs::CS_PQYCbCr2020;
-      }
-      else if (contains(*tok, "bt") || contains(*tok, "709")){
-        *colorspace = pfs::CS_YCbCr709;
-      }
-      else if (contains(*tok, "hlg")){
-        *colorspace = pfs::CS_HLGYCbCr2020;
-      }
-      else if(contains(*tok, "444") || contains(*tok, "420")){
-        *upscale_420 = contains(*tok, "420");
-      }
-    }
-    return true;
-  }
-  catch(...){
-    return false;
-  }
-}
-
 void upscale_420to444( pfs::Array2D *ch )
 {
   const int height = ch->getRows();
@@ -191,20 +118,191 @@ void upscale_420to444( pfs::Array2D *ch )
   }   
 }
 
+void upscale_422to444( pfs::Array2D *ch )
+{
+  const int height = ch->getRows();
+  const int width = ch->getCols();
+  // For odd columns
+  for (int y = 0; y < height; y++){
+    for( int x = 1; x < width-1; x+=2 ) {          
+      (*ch)(x,y) = ((*ch)(x-1,y) + (*ch)(x+1,y))/2.f;     
+    } 
+  }
+}
+
+void parse210Block(unsigned int word, pfs::Array2D *thirdPart, pfs::Array2D *secondPart, pfs::Array2D *firstPart, unsigned int thirdPartX, unsigned int secondPartx, unsigned int firstPartX, unsigned int y, bool ycy){ //last bool denotes the ordering of luma and chroma within a word. not ycy -? cyc
+
+  const unsigned int bitmask = 0x03FF;
+
+  const float luma_offset = 16.f/219.f;
+  const float luma_gain = 1.f/( (float)(1<<(2))*219.f);
+  
+  const float chroma_offset = 128.f/224.f;
+  const float chroma_gain = 1.f/( (float)(1<<(2))*224.f);
+  
+  //now because of the somewhat arbitrary ordering, we don't know beforehand what offset and gain to apply. I'm just trying to make this less messy, but there's definitely a better way. TODO this.
+  float first_last_gain = ycy ? luma_gain : chroma_gain;
+  float first_last_offset = ycy ? luma_offset : chroma_offset;
+  
+  float second_gain = ycy ? chroma_gain : luma_gain;
+  float second_offset = ycy ? chroma_offset : luma_offset;
+  
+  (*thirdPart)(thirdPartX,y) = (float)((word&(bitmask<<20))>>20)*first_last_gain - first_last_offset;
+  (*secondPart)(secondPartx,y) = (float)((word&(bitmask<<10))>>10)*second_gain - second_offset;
+  (*firstPart)(firstPartX,y) = (float)(word&(bitmask))*first_last_gain - first_last_offset;
+}
+
+bool read_v210_yuv( FILE *fh, int width, int height, pfs::Array2D *Ydest,pfs::Array2D *Cbdest, pfs::Array2D *Crdest){
+  assert(sizeof(unsigned int) == 4);
+  
+  //2*width needed because width is needed to store luma, and width is also needed to store both chroma
+  
+  unsigned int paddedLineWidth = (unsigned int) ceil(((float)width)/6.)*4; //in 32 bit words
+  unsigned int * line_buf = new unsigned int[paddedLineWidth];
+
+
+  for( int y = 0; y<height; y++){
+    size_t read = fread( line_buf, sizeof( unsigned int ), paddedLineWidth, fh );
+
+    if( y == 0 && read == 0 ) { // End of file reached
+      delete [] line_buf;
+      return false;
+    }  
+    else if (read != paddedLineWidth){
+      delete [] line_buf;
+      throw pfs::Exception("Error when reading YUV file");
+    }
+    else{
+      //who came up with this insane format
+      //we read 4 words at a time :'(
+      //loop over image pixels, therefore actual data lies in 2* our iteration variable
+      //+= 6 because we write to 6 pixels below
+      unsigned int bufIndex;
+      for (int x = 0; x<width; x+=6){
+        unsigned int word;
+        bufIndex = (x/6)*4;
+        
+        //we store chroma in even cells, will interpolate later
+
+        //first block 
+        word = line_buf[bufIndex];
+        
+        parse210Block(word, Crdest, Ydest, Cbdest, x, x, x, y, false);
+      
+        //second block
+        word = line_buf[bufIndex + 1];
+
+        parse210Block(word, Ydest, Cbdest, Ydest, x+2, x+2, x+1, y, true);
+
+        //third block
+        word = line_buf[bufIndex + 2];
+        
+        parse210Block(word, Cbdest, Ydest, Crdest, x+4, x+3, x+2, y, false);
+
+        //forth block
+        word = line_buf[bufIndex + 3];
+
+        parse210Block(word, Ydest, Crdest, Ydest, x+5, x+4, x+4, y, true);
+        
+      }  
+    }
+  }
+  delete[] line_buf;
+  upscale_422to444(Cbdest);
+  upscale_422to444(Crdest);
+  return true;
+}
+
+void removeCharsFromString(std::string &str, char* charsToRemove ) {
+  for ( unsigned int i = 0; i < strlen(charsToRemove); ++i ) {
+        str.erase( remove(str.begin(), str.end(), charsToRemove[i]), str.end() );
+   }
+}
+
+int parseIntField(std::string i){
+  //parses an integer field. Removes pesky characters like the b in 10b or the p in 1080p
+  removeCharsFromString(i, charsToRemove);
+  return std::atoi(i.data());
+}
+
+//more readable helper method for strings, case insensitive
+bool contains(std::string s1, std::string s2){
+  
+  std::transform(s1.begin(), s1.end(), s1.begin(), ::tolower);
+  std::transform(s2.begin(), s2.end(), s2.begin(), ::tolower);
+  
+  return s1.find(s2) != std::string::npos || s1 == s2;
+}
+
+
+//returns a vector of strings tokenized 
+std::vector<std::string> split(std::string s, std::string delim){
+  return std::vector<std::string>();
+}
+
+bool parseFileName(const char * fileNameIn, int * width,  int * height, int * bitdepth, pfs::ColorSpace * colorspace, ChromaFormat * chromaFormat, int * fps){
+  try{
+    /* Parses the filename, puts the result into fields pointed to by the arguments
+     returns true if all went well, false if there's a malformed filename
+     string should be <prefix>_<width>x<height(p)>_<fps>_<bitdepth(b)>_<colorspace>_<chroma_format>
+     Regex would be nice*/  
+    std::string fileName(fileNameIn); 
+    const std::string delimiter = "_";
+    size_t pos = 0;
+    std::string token;  
+    std::vector<std::string> tokens;
+
+    while ((pos = fileName.find(delimiter)) != std::string::npos) {
+        token = fileName.substr(0, pos);
+        tokens.push_back(token);
+        fileName.erase(0, pos + delimiter.length());
+    }
+    tokens.push_back(fileName);
+
+    for (std::vector<std::string>::iterator tok = tokens.begin() + 1; tok != tokens.end(); ++tok) {
+      
+      if (contains(*tok, "x")){
+        sscanf(tok->c_str(), "%dx%d", width, height);
+      }
+      else if(*tok == "10" || *tok == "8" || *tok == "12" || tok->at(tok->length() - 1) == 'b' || contains(*tok, "bit")){ //last character is a b
+        *bitdepth = parseIntField(*tok); 
+      }
+      else if(*tok == "24" || *tok == "25" || *tok == "50" || *tok == "60" || contains(*tok, "fps")){
+        *fps = parseIntField(*tok);
+      }
+      else if (contains(*tok, "pq") || (contains(*tok, "2020") && !contains(*tok, "hlg"))){
+        *colorspace = pfs::CS_PQYCbCr2020;
+      }
+      else if (contains(*tok, "bt") || contains(*tok, "709")){
+        *colorspace = pfs::CS_YCbCr709;
+      }
+      else if (contains(*tok, "hlg")){
+        *colorspace = pfs::CS_HLGYCbCr2020;
+      }
+      else if(contains(*tok, "444")) *chromaFormat = PL444; 
+      else if(contains(*tok, "420")) *chromaFormat = PL420;
+      else if(contains(*tok, "V210")) *chromaFormat = V210;
+    }
+    return true;
+  }
+  catch(...){
+    return false;
+  }
+}
 
 class YUVReader
 {
   FILE *fh;
   int width, height;
   int bit_depth;
-  bool subsampling_420;
+  ChromaFormat chromaFormat;
   unsigned long int fileSize;
   
   enum ColorSpace { REC709, REC2020 };
   ColorSpace color_space;      
   
 public:
-  YUVReader( FILE *fh, int width, int height, int bit_depth, bool subsampling_420 ) : fh(fh), width( width ), height( height ), bit_depth( bit_depth ), subsampling_420( subsampling_420 )
+  YUVReader( FILE *fh, int width, int height, int bit_depth, ChromaFormat chromaFormat) : fh(fh), width( width ), height( height ), bit_depth( bit_depth ), chromaFormat( chromaFormat )
   {
     //initialize filesize
     fseek(fh, 0L, SEEK_END);
@@ -236,7 +334,7 @@ public:
 //    return fseek(fh, offset, SEEK_CUR);
 //  }
 
-  /* See to a given frame */
+  /* Seek to a given frame */
   bool seekToFrame(int frameNo){
     unsigned long int offset = getFileOffsetFromFrame(frameNo);
     if(offset > fileSize){
@@ -247,15 +345,22 @@ public:
     }
   }
 
+  //gets the position in the file of the specified frame
   unsigned long int getFileOffsetFromFrame(int frameNo){
-    unsigned long int divisor = subsampling_420 ? 4 : 1; //this is the integer divisor of the green and blue channels (they are 4 times smaller if 4:2:0 subsampling is enabled)
+    if(chromaFormat == V210){
+      /*TODO This*/
+      return (unsigned long int) ceil(((float) width)/6)*16*height*frameNo;
+    }
+    else{
+      unsigned long int divisor = chromaFormat == PL420 ? 4 : 1; //this is the integer divisor of the green and blue channels (they are 4 times smaller if 4:2:0 subsampling is enabled)
 
-    if( frameNo <= 0 )
-      throw pfs::Exception( "Invalid frame index. Frame are indexed from 1" );
+      if( frameNo <= 0 )
+        throw pfs::Exception( "Invalid frame index. Frame are indexed from 1" );
 
-    unsigned long int storeFormatSize = bit_depth <= 8 ? sizeof(unsigned char) : sizeof(unsigned short);
-    unsigned long int offset = (frameNo-1) * (/*Luma:*/ width * height * storeFormatSize + /*CrCb:*/ 2*width*height*storeFormatSize/divisor);
-    return offset;
+      unsigned long int storeFormatSize = bit_depth <= 8 ? sizeof(unsigned char) : sizeof(unsigned short);
+      unsigned long int offset = (frameNo-1) * (/*Luma:*/ width * height * storeFormatSize + /*CrCb:*/ 2*width*height*storeFormatSize/divisor);
+      return offset;
+    }
   }
   
   /**
@@ -267,11 +372,16 @@ public:
    */
 
   bool readImage( pfs::Array2D *R, pfs::Array2D *G, pfs::Array2D *B ){  
-    if(bit_depth <= 8){
-      return readImageTyped<unsigned char>(R, G, B);
+    if(chromaFormat == V210){
+      return read_v210_yuv(fh, width, height, R, G, B);
     }
     else{
-      return readImageTyped<unsigned short>(R, G, B);
+      if(bit_depth <= 8){
+        return readImageTyped<unsigned char>(R, G, B);
+      }
+      else{
+        return readImageTyped<unsigned short>(R, G, B);
+      }
     }
   }
 
@@ -285,7 +395,7 @@ public:
       const float gain = 1.f/( (float)(1<<(bit_depth-8))*219.f);    
       if( !read_yuv_channel<T>( fh, width, height, 0, R, gain, offset, 0.f, 1.f ) )  {
         return false;
-      }   
+      } 
       }
       
       { // Read Cb, Cr
@@ -294,7 +404,7 @@ public:
         unsigned int chroma_width = width;
         unsigned int chroma_height = height;
         unsigned int chroma_stride = 0;
-        if (subsampling_420){
+        if (chromaFormat == PL420){
           chroma_width = width/2;
           chroma_height = height/2;
           chroma_stride = 1;  
@@ -305,7 +415,7 @@ public:
 
         if( !read_yuv_channel<T>( fh, chroma_width, chroma_height, chroma_stride, B, gain, offset, -0.5f, 0.5f ) ) 
         throw pfs::Exception( "EOF reached when reading the Cr portion of a frame" );
-        if(subsampling_420){
+        if(chromaFormat == PL420){
           upscale_420to444( B ); 
           upscale_420to444( G );
         }
@@ -434,7 +544,7 @@ void readFrames( int argc, char* argv[] )
   }; // No more frames
 
   int width = -1, height = -1, bitdepth = -1, frame_min = 1, frame_max = INT_MAX, frame_stride = 1, fps = -1;
-  bool upscale_420 = true;
+  ChromaFormat chromaFormat = PL444;
   pfs::ColorSpace colorspace = pfs::CS_INVALID;
 
   VERBOSE_STR << "reading file '" << filename << "'" << std::endl;
@@ -443,7 +553,7 @@ void readFrames( int argc, char* argv[] )
     std::string rawName(filename); //these three lines extract the filename proper, without the containing directory prefix
     long unsigned int substring = rawName.find_last_of("\\/");
     rawName = rawName.substr(substring == std::string::npos ? 0 : substring);
-    bool goodFileName = parseFileName(rawName.c_str(), &width, &height, &bitdepth, &colorspace, &upscale_420, &fps);  
+    bool goodFileName = parseFileName(rawName.c_str(), &width, &height, &bitdepth, &colorspace, &chromaFormat, &fps);  
     if(!goodFileName){
       VERBOSE_STR << "Unable to parse filename: " << filename << "\n";
     }
@@ -464,8 +574,12 @@ void readFrames( int argc, char* argv[] )
   if( opt_colorspace != pfs::CS_INVALID )
     colorspace = opt_colorspace;
   if( opt_chroma_subsampling != NULL ){
-    // bad reading, essentially just force upscaling if they input the subsampling as 420
-    upscale_420 = strcmp(opt_chroma_subsampling, "420") == 0;   
+    //set up our chromat supsampling, defaults to planar 444
+    chromaFormat =
+     strcmp(opt_chroma_subsampling, "420") == 0 ? PL420 :
+     strcmp(opt_chroma_subsampling, "V210") == 0 ? V210 : 
+     PL444;
+ 
   } 
   if( opt_fps > 0 ){
     fps = opt_fps;
@@ -500,7 +614,7 @@ void readFrames( int argc, char* argv[] )
     VERBOSE_STR << "FPS: " << fps << std::endl;
   }
 
-  YUVReader reader( fh, width, height, bitdepth, upscale_420);
+  YUVReader reader( fh, width, height, bitdepth, chromaFormat);
 
   
   int currentFrame = frame_min;
