@@ -39,7 +39,7 @@
 using namespace std;
 
 /// constant to change between radiance and luminance
-#define WHITE_EFFICACY 179.0
+#define WHITE_EFFICACY 179.0f
 
 void readRadianceHeader( FILE *file, int &width, int &height, float &exposure );
 void readRadiance( FILE *file, int width, int height, float exposure,
