@@ -26,7 +26,7 @@
  * 
  * @author Rafal Mantiuk, <mantiuk@gmail.com>
  *
- * $Id: display_function.cpp,v 1.2 2010/07/14 09:45:10 rafm Exp $
+ * $Id: display_function.cpp,v 1.3 2014/04/05 22:04:12 rafm Exp $
  */
 
 #include <math.h>
@@ -35,10 +35,10 @@
 #include <stdlib.h>
 
 #include <pfs.h>
+#include "config.h"
 
 #include "display_function.h"
 
-
 static void removeCommandLineArg( int &argc, char* argv[],
   int firstArgToRemove, int numArgsToRemove );
 
@@ -51,12 +51,13 @@ DisplayFunctionGGBA::DisplayFunctionGGBA( float gamma, float L_max, float L_blac
 
 void DisplayFunctionGGBA::init( float gamma, float L_max, float L_black, float E_amb, float screen_refl )
 {
+	const float PI = 3.1415926535;
   this->gamma = gamma;
   this->L_max = L_max;
   this->L_black = L_black;
   this->E_amb = E_amb;
   this->screen_refl = screen_refl;
-  this->L_offset = L_black + screen_refl/M_PI*E_amb;
+  this->L_offset = L_black + screen_refl/PI*E_amb;
 }
 
 DisplayFunctionGGBA::DisplayFunctionGGBA( const char *predefined )
