@@ -27,6 +27,7 @@
 
 #include <vector>
 #include <algorithm>
+#include <cmath>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -75,7 +76,7 @@ void clamp( pfs::Array2D *array, float min, float max,
     if( v < min ) v = minval;
     else if( v > max ) v = maxval;
     
-    if( !isfinite(v) )
+    if( !std::isfinite(v) )
       v = maxval;
   }    
 }
