@@ -133,7 +133,7 @@ int main( int argc, char* argv[] )
     fprintf( stderr, PROG_NAME " error: %s\n", ex.getMessage() );
     return EXIT_FAILURE;
   }        
-  catch( QuietException  ex ) {
+  catch( QuietException ) {
     return EXIT_FAILURE;
   }
   return EXIT_SUCCESS;
