@@ -22,7 +22,7 @@
  * 
  * @author Rafal Mantiuk, <mantiuk@mpi-sb.mpg.de>
  *
- * $Id: hdrhtml.cpp,v 1.6 2009/03/11 21:45:55 rafm Exp $
+ * $Id: hdrhtml.cpp,v 1.8 2014/06/16 21:50:08 rafm Exp $
  */
 
 #include "hdrhtml.h"
@@ -48,6 +48,8 @@
 
 #include <config.h>
 
+using namespace std;
+
 // ================================================
 //        Parameters controllig the web page 
 // ================================================
@@ -567,7 +569,7 @@ public:
         // Skip white spaces
         while( line_str[pos] == ' ' || line_str[pos] == '\t' ) pos++;
         int new_pos = line_str.find_first_of( ',', pos );
-        int len;
+        size_t len;
         if( new_pos == std::string::npos ) {
           if( k != columns-1 ) {
             std::string full_message( "Missing column data in the file: " );
