@@ -37,6 +37,8 @@
 #include <pfs.h>
 #include <getopt.h>
 
+#include <Magick++.h>
+
 
 #define PROG_NAME "pfsouthdrhtml"
 
@@ -127,6 +129,8 @@ void generate_hdrhtml( int argc, char* argv[] )
  
   bool first_frame = true;
 
+  Magick::InitializeMagick("");
+  
   HDRHTMLSet image_set( page_name, image_dir );
   
   while( true ) {
