@@ -2,7 +2,7 @@
 #define MEX_UTILS_H
 
 #include "mex.h"
-#include "array2d.h"
+#include <array2d.h>
 #include <pfs.h>
 
 namespace pfs {
