@@ -6,9 +6,19 @@ if ~ispc()
     fixed_path = path;
 else
     
-    [pstatus, tmp] = dos('set CYGWIN_HOME');
+    pstatus = dos('set CYGWIN_HOME > nul 2>&1');
     if(pstatus == 1)
-        cygwin_home = 'c:\\cygwin';
+        check_dirs  = { 'c:\\cygwin', 'c:\\cygwin64', 'c:\\cygwin32' };
+        cygwin_home = [];
+        for kk=1:length(check_dirs)
+            if exist( check_dirs{kk}, 'dir' )
+                cygwin_home = check_dirs{kk};
+                break;
+            end
+        end
+        if isempty( cygwin_home )
+            error( 'Cannot find Cygwin home directory. Set CYGWIN_HOME environment variable.' );
+        end
     else
         [pstatus, cygwin_home] = dos('echo %CYGWIN_HOME%');
         cygwin_home = strtrim(cygwin_home); % to remove the final LF
