@@ -18,7 +18,7 @@ function Y = pfs_read_luminance( fileName )
   fclose( fid );
      
 %  try
-    cmd = sprintf( '%spfsin ''%s''%s', pfs_shell(), fileName, pfs_shell( 1 ) );
+    cmd = sprintf( '%spfsin ''%s''%s', pfs_shell(), pfs_fix_path(fileName), pfs_shell( 1 ) );
     fid = pfspopen( cmd, 'r' );
     pin = pfsopen( fid );
     pin = pfsget( pin );
