@@ -18,7 +18,7 @@ function varargout = pfs_read_xyz( fileName )
   end
   fclose( fid );
 
-  fid = pfspopen( sprintf( '%spfsin ''%s''%s', pfs_shell(), fileName, pfs_shell( 1 ) ), 'r' );
+  fid = pfspopen( sprintf( '%spfsin ''%s''%s', pfs_shell(), pfs_fix_path(fileName), pfs_shell( 1 ) ), 'r' );
   pin = pfsopen( fid );
   pin = pfsget( pin );
   pfsclose( pin );
