@@ -24,10 +24,10 @@ else
         % This will remove all references to matlab libraries from the
         % LD_LIBRARY_PATH. pfstools usually do not work with matlab version
         % of the standard libraries 
-        set_ld_path='export LD_LIBRARY_PATH=`echo $LD_LIBRARY_PATH | sed "y/:/\n/" | grep -v "matlab" | sed ":beg;N;s/\n/:/;t beg;"`';
+        set_ld_path='export LD_LIBRARY_PATH=`echo $LD_LIBRARY_PATH | sed "y/:/\n/" | grep --ignore-case -v "matlab" | sed ":beg;N;s/\n/:/;t beg;"`';
         
         if( strcmp( computer, 'MACI64' ) || strcmp( computer, 'MACI32' ) )
-            set_ld_path=cat( 2, set_ld_path, '; export DYLD_FRAMEWORK_PATH=`echo $DYLD_FRAMEWORK_PATH | sed "y/:/\n/" | grep -v "matlab" | sed ":beg;N;s/\n/:/;t beg;"`' );
+            set_ld_path=cat( 2, set_ld_path, '; export DYLD_FRAMEWORK_PATH=`echo $DYLD_FRAMEWORK_PATH | sed "y/:/\n/" | grep --ignore-case -v "matlab" | sed ":beg;N;s/\n/:/;t beg;"`' );
         end
         
         % Start shell, cd to the working directory and remove matlab
