@@ -34,6 +34,10 @@
 #include <string>
 #include <pfs.h>
 
+#if OCTAVE_MAJOR_VERSION < 4 || (OCTAVE_MAJOR_VERSION == 4 && OCTAVE_MINOR_VERSION < 4)
+  #define isstruct is_map
+#endif
+
 #define SCRIPT_NAME "pfsclose"
 
 static const char *helpString =
@@ -47,7 +51,7 @@ DEFUN_DLD( pfsclose, args, , helpString )
 
   // Get arguments and check if they are legal
     
-  if( nargin != 1 || !args(0).is_map() )
+  if( nargin != 1 || !args(0).isstruct() )
   {
     error( SCRIPT_NAME ": Improper usage!");
     return retval;
