@@ -29,12 +29,21 @@
 #include "pfs.h"
 #include <assert.h>
 #include <list>
+#include <algorithm>
 
 #include <iostream>
 
 namespace pfs 
 {
 
+const float Lmax = 10000.f;
+const float n    = 0.15930175781250000f;
+const float m    = 78.843750000000000f; 
+const float c1   = 0.83593750000000000f;  
+const float c2   = 18.851562500000000f; 
+const float c3   = 18.687500000000000f; 
+
+
 //--- 7 digits approximation of precise values
 static const float rgb2xyzD65Mat[3][3] =
 { { 0.412424f, 0.357579f, 0.180464f },
@@ -46,6 +55,32 @@ static const float xyz2rgbD65Mat[3][3] =
   { -0.969257f,  1.875995f,  0.041555f },
   {  0.055636f, -0.203996f,  1.057069f } };
 
+static const float ycbcr2rgb_bt709[3][3] = 
+{ { 1.f,  0.f,       1.5748f },
+  { 1.f, -0.18733f, -0.46813f },
+  { 1.f,  1.85563f,  0.f } };
+
+static const float ycbcr2rgb_bt2020[3][3] = 
+{ { 1.f,  0.f,       1.47460f },
+  { 1.f, -0.16455f, -0.57135f },
+  { 1.f,  1.88140f,  0.f } };
+
+static const float rgb2020_to_xyz[3][3] = 
+{ { 0.6370f, 0.1446f, 0.1689f },
+  { 0.2627f, 0.6780f, 0.0593f }, 
+  { 0.0000f, 0.0281f, 1.0610f } };
+
+static const float xyz_to_rgb2020[3][3] = 
+{{1.71650250836063f, -0.355584689096764f, -0.25337521357085f},
+ {-0.666625609145029f, 1.61644656652221f, 0.0157754797265114f},
+ {0.017655211703087f, -0.0428106960596362, 0.942089263920533f}};
+
+static const float rgb2020_to_ycbcr[3][3] = 
+{ { 0.2627f,        0.6780f,      0.0593f },
+  { -0.139630063f, -0.360369937f, 0.5f}, 
+  { 0.5f,          -0.459785705, -0.040214295 } };
+
+
 // //--- precise values for matrix convertion (above float precission)
 // static const float rgb2xyzD65Mat[3][3] =
 // { { 0.412424,  0.357579, 0.180464 },
@@ -209,6 +244,137 @@ static void transformXYZ2RGB( const Array2D *inC1, const Array2D *inC2, const Ar
   multiplyByMatrix( inC1, inC2, inC3, outC1, outC2, outC3, xyz2rgbD65Mat );
 }
 
+float pq2linFunction(float in){
+  double v_t = powf( std::max(in,0.f), 1.f/m );
+  return Lmax * powf(std::max( v_t-c1, (double) 0.f )/(c2-c3*v_t), 1.f/n);
+}
+
+static void transform_pq2lin( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  int imgSize = inC1->getRows()*inC1->getCols();
+  for( int index = 0; index < imgSize ; index++ ) {
+    (*outC1)(index) = pq2linFunction((*inC1)(index));
+    (*outC2)(index) = pq2linFunction((*inC2)(index));
+    (*outC3)(index) = pq2linFunction((*inC3)(index));
+  }
+}
+
+float lin2pqFunction(float in){
+  in = std::max(in, 0.0f);
+  double L_n_o = pow(((double) in)/Lmax, n);
+  return pow((c1+c2*L_n_o)/(1+c3*L_n_o), m);
+}
+
+static void transform_lin2pq( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+
+  int imgSize = inC1->getRows()*inC1->getCols();
+  
+  for (int i = 0; i< imgSize; i++){
+    (*outC1)(i) = lin2pqFunction((*inC1)(i));
+    (*outC2)(i) = lin2pqFunction((*inC2)(i));
+    (*outC3)(i) = lin2pqFunction((*inC3)(i));
+  }
+}
+
+float HLG_OETF(float in){
+  float a = 0.17883277;
+  float b = 1-4*a;
+        float c = 0.5-a*log(4*a);
+  float r = 0.5;
+  float normE = 10000.0/12.0;
+  in = in/normE;
+  if(in <= 1.0){
+    return r*sqrt(in);
+  }
+  else{
+    return a*log(in - b) + c;
+  }
+}
+
+float HLG_inverse_OETF(float in){
+  float a = 0.17883277;
+  float b = 1-4*a;
+  float c = 0.5-a*log(4*a);
+  float r = 0.5;
+  float normE = 10000.0/12.0;
+  float rtn = 0;
+  if(in<=r){
+    rtn = pow(in/r,2);
+  }
+  else{
+    rtn = exp((in-c)/a) + b;
+  }
+  return rtn*normE;
+
+}
+
+static void hlg2lin( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3, const float L_w ){
+  int imgSize = inC1->getRows()*inC1->getCols();
+  for (int i = 0; i<imgSize; i++){
+    (*outC1)(i) = HLG_inverse_OETF((*inC1)(i));
+
+    (*outC2)(i) = HLG_inverse_OETF((*inC2)(i));
+
+          (*outC3)(i) = HLG_inverse_OETF((*inC3)(i)); 
+  }
+}
+
+static void lin2hlg(const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  int imgSize = inC1->getCols()*inC1->getRows();
+  for(int i = 0; i<imgSize; i++){
+    (*outC1)(i) = HLG_OETF((*inC1)(i));
+    (*outC2)(i) = HLG_OETF((*inC2)(i));
+    (*outC3)(i) = HLG_OETF((*inC3)(i));
+  }
+}
+
+
+static void transformYCbCr709_to_SRGB( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  multiplyByMatrix( inC1, inC2, inC3, outC1, outC2, outC3, ycbcr2rgb_bt2020 );  
+}
+
+static void transformSRGB_to_YCbCr709( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  multiplyByMatrix( inC1, inC2, inC3, outC1, outC2, outC3, rgb2020_to_ycbcr); 
+}
+
+static void transformPQYCbCr2020_to_RGB2020( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  multiplyByMatrix( inC1, inC2, inC3, outC1, outC2, outC3, ycbcr2rgb_bt2020 );  
+  transform_pq2lin( inC1, inC2, inC3, outC1, outC2, outC3 );
+}
+
+static void transformRGB2020_to_PQYCbCr2020( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  transform_lin2pq( inC1, inC2, inC3, outC1, outC2, outC3 );
+  multiplyByMatrix( inC1, inC2, inC3, outC1, outC2, outC3, rgb2020_to_ycbcr);  
+}
+
+static void transformRGB2020_to_HLGYCbCr(const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3){
+  lin2hlg(inC1, inC2, inC3, outC1, outC2, outC3);
+  multiplyByMatrix(inC1, inC2, inC3, outC1, outC2, outC3, rgb2020_to_ycbcr);
+  
+}
+
+static void transformHLGYCbCr_to_RGB2020(const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3){
+  multiplyByMatrix(inC1, inC2, inC3, outC1, outC2, outC3, ycbcr2rgb_bt2020);
+  hlg2lin(inC1, inC2, inC3, outC1, outC2, outC3, 10000.0);
+}
+
+static void transformRGB2020_to_XYZ( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  multiplyByMatrix( inC1, inC2, inC3, outC1, outC2, outC3, rgb2020_to_xyz );  
+}
+
+static void transformXYZ_to_RGB2020( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 )
+{
+  multiplyByMatrix( inC1, inC2, inC3, outC1, outC2, outC3, xyz_to_rgb2020 );  
+}
+
+
 typedef void(*CSTransformFunc)( const Array2D *inC1, const Array2D *inC2, const Array2D *inC3, Array2D *outC1, Array2D *outC2, Array2D *outC3 );
 
 struct CSTransEdge
@@ -219,26 +385,47 @@ struct CSTransEdge
   CSTransformFunc func;
 };
 
-CSTransEdge TN_XYZRGB = { NULL, CS_XYZ, CS_RGB, transformXYZ2RGB };
+CSTransEdge TN_XYZ_to_RGB2020 = { NULL, CS_XYZ, CS_RGB2020, transformXYZ_to_RGB2020 };
+CSTransEdge TN_XYZRGB = { &TN_XYZ_to_RGB2020, CS_XYZ, CS_RGB, transformXYZ2RGB };
 CSTransEdge TN_XYZYUV = { &TN_XYZRGB, CS_XYZ, CS_YUV, transformXYZ2Yuv };
 CSTransEdge TN_XYZYxy = { &TN_XYZYUV, CS_XYZ, CS_Yxy, transformXYZ2Yxy };
 CSTransEdge TN_XYZSRGB = { &TN_XYZYxy, CS_XYZ, CS_SRGB, transformXYZ2SRGB };
 
+
 CSTransEdge TN_RGBXYZ = { NULL, CS_RGB, CS_XYZ, transformRGB2XYZ };
 
-CSTransEdge TN_SRGBXYZ = { NULL, CS_SRGB, CS_XYZ, transformSRGB2XYZ };
+CSTransEdge TN_SRGB_to_YCbCr709 = { NULL, CS_SRGB, CS_YCbCr709, transformSRGB_to_YCbCr709 };
+CSTransEdge TN_SRGBXYZ = { &TN_SRGB_to_YCbCr709, CS_SRGB, CS_XYZ, transformSRGB2XYZ };
 
 CSTransEdge TN_YUV2XYZ = { NULL, CS_YUV, CS_XYZ, transformYuv2XYZ };
 
 CSTransEdge TN_Yxy2XYZ = { NULL, CS_Yxy, CS_XYZ, transformYxy2XYZ };
 
+CSTransEdge TN_PQYCbCr2020_to_RGB2020 = {NULL, CS_PQYCbCr2020, CS_RGB2020, transformPQYCbCr2020_to_RGB2020 };
+
+CSTransEdge TN_RGB2020_to_HLGYCbCr = {NULL, CS_RGB2020, CS_HLGYCbCr2020, transformRGB2020_to_HLGYCbCr};
+
+CSTransEdge TN_HLGYCbCr_to_RGB2020 = {NULL, CS_HLGYCbCr2020, CS_RGB2020, transformHLGYCbCr_to_RGB2020};
+
+CSTransEdge TN_YCbCr709_to_SRGB = { NULL, CS_YCbCr709, CS_SRGB, transformYCbCr709_to_SRGB };
+
+
+CSTransEdge TN_RGB2020_to_PQYCbCr2020 = { &TN_RGB2020_to_HLGYCbCr, CS_RGB2020, CS_PQYCbCr2020, transformRGB2020_to_PQYCbCr2020 };
+CSTransEdge TN_RGB2020 = { &TN_RGB2020_to_PQYCbCr2020, CS_RGB2020, CS_XYZ, transformRGB2020_to_XYZ };
+
+// The order of elements must be exactly the same as enum pfs::ColorSpace
 CSTransEdge *CSTransGraph[] =
 {
   &TN_XYZSRGB,
   &TN_RGBXYZ,
   &TN_SRGBXYZ,
   &TN_YUV2XYZ,
-  &TN_Yxy2XYZ
+  &TN_Yxy2XYZ,
+  NULL, // This is for CS_INVALID
+  &TN_PQYCbCr2020_to_RGB2020,
+  &TN_YCbCr709_to_SRGB,
+  &TN_HLGYCbCr_to_RGB2020,
+  &TN_RGB2020
 };
 
 
@@ -286,7 +473,7 @@ void transformColorSpace( ColorSpace inCS,
 
   if( !found ) {
     // TODO: All transforms should be supported
-    throw Exception( "Not supported color tranform" );
+    throw Exception( "Not supported color transform" );
   } else {
     // Reverse path
     std::list<CSTransEdge *> step;
@@ -307,10 +494,6 @@ void transformColorSpace( ColorSpace inCS,
       else
         (*it)->func( outC1, outC2, outC3, outC1, outC2, outC3 );      
     }
-      
   }
-    
 }
-
-
 }
