@@ -32,9 +32,7 @@
 #endif
 
 // This does not seem to be needed under Cygwin
-//#if defined(_MSC_VER) || defined(__CYGWIN__)
-
-#if defined(_MSC_VER)
+#if defined(_MSC_VER) || defined(__MINGW64__) || defined(__MINGW32__)
 #include <io.h>
 #define HAVE_SETMODE
 #endif
