@@ -543,7 +543,7 @@ namespace pfs
   enum ColorSpace
     {
       CS_XYZ = 0,         ///< Absolute XYZ space, reference white - D65, Y is calibrated luminance in cd/m^2
-      CS_RGB,             ///< Absolute XYZ space, reference white - D65
+      CS_RGB,             ///< Absolute XYZ space, reference white - D65, Rec. 709
       CS_SRGB,            ///< sRGB color space for LDR images (see
                           ///www.srgb.com). The possible pixel values
                           ///for R, G and B channel should be within
@@ -551,7 +551,12 @@ namespace pfs
                           ///range will be clamped). Peak luminance
                           ///level of the display is 80cd/m^2.
       CS_YUV,             ///< Perceptually uniform u and v color coordinates, Y is calibrated luminance in cd/m^2
-      CS_Yxy,              ///< Luminance and normalized chromacities (x=X/(X+Y+Z), y=Y/(X+Y+Z))
+      CS_Yxy,             ///< Luminance and normalized chromacities (x=X/(X+Y+Z), y=Y/(X+Y+Z))
+	  CS_INVALID,         ///< For convenience	     
+	  CS_PQYCbCr2020,	  ///< MPEG YCbCr with the PQ transfer function and Rec. 2020 colorspace
+	  CS_YCbCr709,	      ///< MPEG YCbCr with the sRGB non-linearity and Rec. 709 colorspace
+	  CS_HLGYCbCr2020,
+	  CS_RGB2020,		  ///< Rec 2020 wide color gamut RGB colorspace
       CS_LAST             ///< For internal purposes only
     };
 
