@@ -1,20 +1,45 @@
-#ifndef PICTUREIO_H
-#define PICTUREIO_H
+/**
+ * @brief PFS library - additional utilities
+ *
+ * This file is a part of PFSTOOLS package.
+ * ----------------------------------------------------------------------
+ * Copyright (C) 2006 Radoslaw Mantiuk
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * ----------------------------------------------------------------------
+ *
+ * @author Radoslaw Mantiuk, <radoslaw.mantiuk@gmail.com>
+ * @author Oliver Barth <obarth@mpi-inf.mpg.de>
+ */
+
+#ifndef PICTURE_IO_H
+#define PICTURE_IO_H
 
 #include <list>
-#include <pfs.h>
+
+#include "pfs.h"
 
 #define MAX_FRAMES_IN_MEMORY 10
 
-class pfs::Frame;
 enum LumMappingMethod {
 			MAP_LINEAR,
 			MAP_GAMMA1_4,
 			MAP_GAMMA1_8,
 			MAP_GAMMA2_2,
 			MAP_GAMMA2_6,
-			MAP_LOGARITHMIC,
-			MAP_REINHARD
+			MAP_LOGARITHMIC
 			};
 			
 static const char* lumMappingName[] = { // must be consistent with LumMappingMethod
@@ -23,8 +48,7 @@ static const char* lumMappingName[] = { // must be consistent with LumMappingMet
 			"Gamma 1.8",
 			"Gamma 2.2",
 			"Gamma 2.6",
-			"Logarithmic",
-			"Photographic"
+			"Logarithmic"
 			};
 
 
@@ -43,14 +67,17 @@ class PictureIO
 
 		unsigned char* data;
 		LumMappingMethod imageMappingMethod;
-		float minLuminance, maxLuminance;		
+		float minLuminance, maxLuminance;
+		int is_show_clipping ;
 
 	public:	
 		const char* CHANNEL_XYZ;
 		const char* CHANNEL_X;
 		const char* CHANNEL_Y;
 		const char* CHANNEL_Z;	
-	
+
+		int abort_update ;
+
 		PictureIO(LumMappingMethod mappingMethod, float minLuminance, float maxLumuminance);
 		~PictureIO();
 		void gotoNextFrame();
@@ -94,11 +121,16 @@ class PictureIO
 		std::vector<const char*> getChannelNames();
 		
 		int getRawData( int x, int y, float& XX, float& YY, float& ZZ);
+		void updateMapping( void CB(void));
 		
+		int get_is_show_clipping() const ;
+		void set_is_show_clipping(int is_show_clipping) ;
+		inline int fastMap( LumMappingMethod mappingMethod, float v, float minValue, float maxValue, float ranger) ;
+		inline int fastCheck( LumMappingMethod mappingMethod, float v, float minValue, float maxValue, float ranger) ;
+
 	private:
 		bool hasColorChannels( pfs::Frame *frame );
-		void updateMapping( void);
-		inline int binarySearchPixels( float lum, const float *lumMap, const int lumSize);
+		inline int binarySearchPixels(const float lum, const float *lumMap, const int lumSize);
 		float getInverseMapping( LumMappingMethod mappingMethod, float v, float minValue, float maxValue );
 		bool readNextFrame();
 };
