@@ -25,7 +25,7 @@
  *
  * @author Rafal Mantiuk, <mantiuk@mpi-sb.mpg.de>
  *
- * $Id: luminancerange_widget.h,v 1.3 2011/03/21 16:09:16 rafm Exp $
+ * $Id: luminancerange_widget.h,v 1.4 2013/12/21 19:42:29 rafm Exp $
  */
 
 #include <QMouseEvent>
@@ -51,7 +51,7 @@ protected:
   void paintEvent( QPaintEvent * );
   void mouseMoveEvent( QMouseEvent * );
   void mousePressEvent( QMouseEvent * me );
-  void mouseReleaseEvent( QMouseEvent * me );
+  void mouseReleaseEvent(QMouseEvent *);
 
   float draggedMin();
   float draggedMax();
@@ -83,13 +83,13 @@ private:
     };
   DragMode dragMode;
   
-
-  bool showVP;
   float valuePointer;
 
   Histogram *histogram;
   const pfs::Array2D *histogramImage;
 
+  bool showVP;
+
   QRect getPaintRect() const;
   
 public:
