@@ -460,7 +460,7 @@ void PFSViewMainWin::updatePointerValue()
       
     } else {                    // Single channel
       const QString &name = channelSelection->itemText( channelSel );
-      sprintf( pointerValueStr, "%s=%07.4g", (const char*)name.toAscii(), pv.value[0] );
+      sprintf( pointerValueStr, "%s=%07.4g", (const char*)name.toLatin1(), pv.value[0] );
       label += pointerValueStr;
       lumRange->showValuePointer( log10(pv.value[0]) );
     }    
@@ -547,7 +547,7 @@ void PFSViewMainWin::setChannelSelection( int selectedChannel )
   if( pfsView->hasColorChannels() && selectedChannel < scLabelsCount ) {
     pfsView->setVisibleChannel( NULL ); // Color
   } else {
-    pfsView->setVisibleChannel( (const char*)name.toAscii() );
+    pfsView->setVisibleChannel( (const char*)name.toLatin1() );
   }
 
   updateMenuEnable( );
