@@ -143,6 +143,8 @@ private:
   NegativeTreatment negativeTreatment;
   LumMappingMethod mappingMethod;
 
+  bool fit_to_content_mode; // If true, newly loaded frames are resized to fit the window  
+  
   float zoom;
 
   pfs::Array2D *workArea[3];
@@ -166,6 +168,12 @@ public:
   
   bool hasColorChannels( pfs::Frame *frame = NULL );
 
+/*  void setFitToContentMode( bool active )
+  {
+    fit_to_content_mode = active;
+    }*/
+  
+
   float getZoom() const
     {
       return zoom;
